@@ -5,7 +5,7 @@ from cognite.experimental.data_classes import ContextualizationJob, ResourceTypi
 
 
 class ResourceTypingAPI(ContextModelAPI):
-    _RESOURCE_PATH = "/context/resource_typing"
+    _RESOURCE_PATH = ResourceTypingModel._RESOURCE_PATH
     _MODEL_CLASS = ResourceTypingModel
 
     def fit(
@@ -13,7 +13,7 @@ class ResourceTypingAPI(ContextModelAPI):
         items: List[TypingFitData],
         algorithm: str = "open_set_nearest_neighbors",
         targets_to_classify: List[str] = None,
-    ) -> "Task[ContextualizationModel]":
+    ) -> ResourceTypingModel:
         """Fit entity matching model.
 
         Args:
@@ -22,7 +22,7 @@ class ResourceTypingAPI(ContextModelAPI):
             targets_to_classify (str): The classes the model will try to classify items into. All other targets will be treated as a single 'other' class.
 
         Returns:
-            Task[ResourceTypingModel]: Task which waits for the model to be completed."""
+            ResourceTypingModel: Resulting queued model."""
         return self._fit_model(
             items=ResourceTypingModel.format_items(items), algorithm=algorithm, targets_to_classify=targets_to_classify
         )
