@@ -3,6 +3,7 @@ from typing import Callable, Dict, Optional, Union
 
 from cognite.client._api.datapoints import DatapointsAPI
 from cognite.client._api.files import FilesAPI
+from cognite.client._api_client import APIClient
 from cognite.client._cognite_client import CogniteClient as Client
 from cognite.experimental._api.assets import ExperimentalAssetsAPI
 from cognite.experimental._api.entity_extraction import EntityExtractionAPI
@@ -30,6 +31,16 @@ class ExperimentalFilesApi(FilesAPI):
         self.unstructured = GrepAPI(self._config, api_version="playground", cognite_client=self)
 
 
+APIClient.RETRYABLE_POST_ENDPOINTS |= {
+    "/timeseries/synthetic/query",
+    "/files/unstructured/search",
+    "/files/unstructured/downloadlink/parsed",
+}
+APIClient.RETRYABLE_POST_ENDPOINTS |= {
+    f"/{api}/{endpoint}" for api in ["types", "labels", "relationships"] for endpoint in ["list", "byids", "search"]
+}
+
+
 class CogniteClient(Client):
     """Initializes cognite client, with experimental extensions.
 
