@@ -1,7 +1,7 @@
 import copy
 import math
 import time
-from typing import Iterable, List, Union
+from typing import Dict, Iterable, List, Optional, Tuple, Union
 
 from typing_extensions import TypedDict
 
@@ -11,18 +11,36 @@ from cognite.experimental.exceptions import ModelFailedException
 
 
 class ContextualizationJob(CogniteResource):
-    def __init__(self, job_id=None, status=None, error_message=None, status_path=None, cognite_client=None, **kwargs):
+    def __init__(
+        self,
+        job_id=None,
+        status=None,
+        error_message=None,
+        request_timestamp=None,
+        start_timestamp=None,
+        status_timestamp=None,
+        status_path=None,
+        cognite_client=None,
+        **kwargs,
+    ):
         """Data class for the result of a contextualization job. All keys in the body become snake-cased variables in the class (e.g. `items`, `svg_url`)"""
         self.job_id = job_id
         self.status = status
+        self.request_timestamp = request_timestamp
+        self.start_timestamp = start_timestamp
+        self.status_timestamp = status_timestamp
         self.error_message = error_message
         self._cognite_client = cognite_client
         self._result = None
+        self._status_path = status_path
 
     def update_status(self) -> str:
         """Updates the model status and returns it"""
-        data = self._cognite_client.entity_matching._get(f"{self.status_path}{self.job_id}").json()  # any playground
+        data = self._cognite_client.entity_matching._get(f"{self._status_path}{self.job_id}").json()  # any playground
         self.status = data["status"]
+        self.status_timestamp = data.get("statusTimestamp")
+        self.start_timestamp = data.get("startTimestamp")
+        self.request_timestamp = self.request_timestamp or data.get("requestTimestamp")
         self.error_message = data.get("errorMessage")
         self._result = {k: v for k, v in data.items() if k not in {"status", "jobId", "errorMessage"}}
         return self.status
@@ -55,7 +73,7 @@ class ContextualizationJob(CogniteResource):
     @staticmethod
     def _load_with_status(data, status_path, cognite_client):
         obj = ContextualizationJob._load(data, cognite_client=cognite_client)
-        obj.status_path = status_path
+        obj._status_path = status_path
         return obj
 
 
@@ -64,9 +82,21 @@ class ContextualizationModel(CogniteResource):
 
     _STATUS_PATH = None
 
-    def __init__(self, model_id=None, status=None, error_message=None, cognite_client=None):
+    def __init__(
+        self,
+        model_id=None,
+        status=None,
+        error_message=None,
+        request_timestamp=None,
+        start_timestamp=None,
+        status_timestamp=None,
+        cognite_client=None,
+    ):
         self.model_id = model_id
         self.status = status
+        self.request_timestamp = request_timestamp
+        self.start_timestamp = start_timestamp
+        self.status_timestamp = status_timestamp
         self.error_message = error_message
         self._cognite_client = cognite_client
 
@@ -78,13 +108,13 @@ class ContextualizationModel(CogniteResource):
             self.error_message,
         )
 
-    def predict(self, asynchronous=False, **kwargs):
-        raise NotImplementedError(f"Predict not implemented for {self.__class__}")
-
     def update_status(self) -> str:
         """Updates the model status and returns it"""
         data = self._cognite_client.entity_matching._get(f"{self._STATUS_PATH}{self.model_id}").json()
         self.status = data["status"]
+        self.status_timestamp = data.get("statusTimestamp")
+        self.start_timestamp = data.get("startTimestamp")
+        self.request_timestamp = self.request_timestamp or data.get("requestTimestamp")
         self.error_message = data.get("errorMessage")
         return self.status
 
@@ -114,6 +144,47 @@ class EntityMatchingModel(ContextualizationModel):
         self.wait_for_completion()
         return self._cognite_client.entity_matching._run_job(job_path=f"/{self.model_id}/predict", items=list(entities))
 
+    def predict_ml(
+        self, match_from: Optional[List[Dict]] = None, match_to: Optional[List[Dict]] = None, num_matches=1
+    ) -> ContextualizationJob:
+        """Predict entity matching.
+
+        Args:
+            match_from: entities to match from, does not need an 'id' field. Tolerant to passing more than is needed or used (e.g. json dump of time series list). If omitted, will use data from fit.
+            match_to: entities to match to, does not need an 'id' field.  Tolerant to passing more than is needed or used. If omitted, will use data from fit.
+            num_matches (int): number of matches to return for each item.
+        Returns:
+            ContextualizationJob: object which can be used to wait for and retrieve results."""
+        self.wait_for_completion()
+        return self._cognite_client.entity_matching._run_job(
+            job_path=f"/{self.model_id}/predictml",
+            match_from=self.dump_entities(match_from),
+            match_to=self.dump_entities(match_to),
+            num_matches=num_matches,
+        )
+
+    def refit_ml(self, true_matches: List[Tuple[int, int]]) -> "EntityMatchingModel":
+        """Re-fits an entity matching on updated data.
+
+        Args:
+            true_matches: Updated known valid matches given as a list of (id_from,id_to).
+        Returns:
+            EntityMatchingModel: new model refitted to ."""
+        self.wait_for_completion()
+        return self._cognite_client.entity_matching._fit_model(
+            model_path=f"/{self.model_id}/refitml", true_matches=true_matches
+        )
+
+    @staticmethod
+    def dump_entities(entities: List[Union[Dict, CogniteResource]]) -> Optional[List[Dict]]:
+        if entities:
+            return [
+                {k: v for k, v in e.dump(camel_case=True).items() if isinstance(v, str) or k == "id"}
+                if isinstance(e, CogniteResource)
+                else e
+                for e in entities
+            ]
+
 
 class TypingPredictData(TypedDict):
     data: List[str]
@@ -130,9 +201,9 @@ class ResourceTypingModel(ContextualizationModel):
 
     @staticmethod
     def format_items(
-        items: List[Union[TypingFitData, TypingPredictData]]
+        items: Iterable[Union[TypingFitData, TypingPredictData]]
     ) -> List[Union[TypingFitData, TypingPredictData]]:
-        items = copy.deepcopy(items)
+        items = copy.deepcopy(list(items))
         for item in items:
             item["data"] = ["" if isinstance(x, float) and math.isnan(x) else x for x in item["data"]]
         return items
