@@ -1,5 +1,5 @@
 import time
-from typing import Dict, List, Union
+from typing import Dict, List, Optional, Union
 
 from cognite.client.data_classes._base import CogniteResource, CogniteResourceList
 
@@ -50,18 +50,59 @@ class Function(CogniteResource):
         self.error = error
         self._cognite_client = cognite_client
 
-    def call(self, data=None, asynchronous: bool = False):
-        return self._cognite_client.functions.call(id=self.id, data=data, asynchronous=asynchronous)
+    def call(self, data=None, wait: bool = True) -> "FunctionCall":
+        """`Call this particlar function. <https://docs.cognite.com/api/playground/#operation/post-api-playground-projects-project-functions-function_name-call>`_
 
-    def list_calls(self):
-        return self._cognite_client.functions.calls.list(function_id=self.id)
+        Args:
+            data (Union[str, dict], optional): Input data to the function (JSON serializable). This data is passed deserialized into the function through one of the arguments called data.
+            wait (bool): Wait until the function call is finished. Defaults to True.
 
-    def list_schedules(self):
+        Returns:
+            FunctionCall: A function call object.
+        """
+        return self._cognite_client.functions.call(id=self.id, data=data, wait=wait)
+
+    def list_calls(
+        self,
+        status: Optional[str] = None,
+        schedule_id: Optional[int] = None,
+        start_time: Optional[Dict[str, int]] = None,
+        end_time: Optional[Dict[str, int]] = None,
+    ) -> "FunctionCallList":
+        """List all calls to this function.
+
+        Args:
+            status (str, optional): Status of the call. Possible values ["Running", "Failed", "Completed", "Timeout"].
+            schedule_id (int, optional): Schedule id from which the call belongs (if any).
+            start_time ([Dict[str, int], optional): Start time of the call. Possible keys are `min` and `max`, with values given as time stamps in ms.
+            end_time (Dict[str, int], optional): End time of the call. Possible keys are `min` and `max`, with values given as time stamps in ms.
+
+        Returns:
+            FunctionCallList: List of function calls
+        """
+        return self._cognite_client.functions.calls.list(
+            function_id=self.id, status=status, schedule_id=schedule_id, start_time=start_time, end_time=end_time
+        )
+
+    def list_schedules(self) -> "FunctionSchedulesList":
+        """`List all schedules associated with this function. <https://docs.cognite.com/api/playground/#operation/get-api-playground-projects-project-functions-schedules>`_
+
+        Returns:
+            FunctionSchedulesList: List of function schedules
+        """
         all_schedules = self._cognite_client.functions.schedules.list()
         function_schedules = filter(lambda f: f.function_external_id == self.external_id, all_schedules)
         return list(function_schedules)
 
-    def retrieve_call(self, id: int):
+    def retrieve_call(self, id: int) -> "FunctionCall":
+        """`Retrieve call by id. <https://docs.cognite.com/api/playground/#operation/get-api-playground-projects-project-functions-function_name-calls-call_id>`_
+
+        Args:
+            id (int): ID of the call.
+
+        Returns:
+            FunctionCall: Function call.
+        """
         return self._cognite_client.functions.calls.retrieve(call_id=id, function_id=self.id)
 
 
@@ -117,8 +158,8 @@ class FunctionCall(CogniteResource):
         id (int): A server-generated ID for the object.
         start_time (int): The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.
         end_time (int): The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.
-        response (str): Response from the function. The function must return a JSON serializable object or nothing.
         status (str): Status of the function call ("Running" or "Completed").
+        schedule_id (int): The schedule id belonging to the call.
         error (dict): Error from the function call. It contains an error message and the stack trace.
         cognite_client (CogniteClient): An optional CogniteClient to associate with this data class.
     """
@@ -128,8 +169,8 @@ class FunctionCall(CogniteResource):
         id: int = None,
         start_time: int = None,
         end_time: int = None,
-        response: str = None,
         status: str = None,
+        schedule_id: int = None,
         error: dict = None,
         function_id: int = None,
         cognite_client=None,
@@ -137,20 +178,37 @@ class FunctionCall(CogniteResource):
         self.id = id
         self.start_time = start_time
         self.end_time = end_time
-        self.response = response
         self.status = status
+        self.schedule_id = schedule_id
         self.error = error
-        self._function_id = function_id
+        self.function_id = function_id
         self._cognite_client = cognite_client
 
-    def get_logs(self):
-        return self._cognite_client.functions.calls.get_logs(call_id=self.id, function_id=self._function_id)
+    def get_response(self):
+        """Retrieve the response from this function call.
 
-    def update(self):
-        latest = self._cognite_client.functions.calls.retrieve(call_id=self.id, function_id=self._function_id)
+        Returns:
+            Response from the function call.
+        """
+        return self._cognite_client.functions.calls.get_response(call_id=self.id, function_id=self.function_id)
+
+    def get_logs(self) -> "FunctionCallLog":
+        """`Retrieve logs for this function call. <https://docs.cognite.com/api/playground/#operation/get-api-playground-projects-project-functions-function_name-calls>`_
+
+        Returns:
+            FunctionCallLog: Log for the function call.
+        """
+        return self._cognite_client.functions.calls.get_logs(call_id=self.id, function_id=self.function_id)
+
+    def update(self) -> None:
+        """Update the funciton call object. Can be useful if the call was made with wait=False.
+
+        Returns:
+            None
+        """
+        latest = self._cognite_client.functions.calls.retrieve(call_id=self.id, function_id=self.function_id)
         self.status = latest.status
         self.end_time = latest.end_time
-        self.response = latest.response
         self.error = latest.error
 
     def wait(self):
@@ -158,25 +216,11 @@ class FunctionCall(CogniteResource):
             self.update()
             time.sleep(1.0)
 
-    @classmethod
-    def _load(cls, resource: Union[Dict, str], function_id: int = None, cognite_client=None):
-        instance = super()._load(resource, cognite_client=cognite_client)
-        if function_id:
-            instance._function_id = function_id
-        return instance
-
 
 class FunctionCallList(CogniteResourceList):
     _RESOURCE = FunctionCall
     _ASSERT_CLASSES = False
 
-    @classmethod
-    def _load(cls, resource: Union[List, str], function_id: int, cognite_client=None):
-        instance = super()._load(resource, cognite_client=cognite_client)
-        for obj in instance:
-            obj._function_id = function_id
-        return instance
-
 
 class FunctionCallLogEntry(CogniteResource):
     """A log entry for a function call.
